/*
 * This file is part of unity-2d
 *
 * Copyright 2010-2011 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 1.1
import Unity2d 1.0
import "launcher"
import "common/units.js" as Units

Item {
    id: shell
    Accessible.name: "shell"

    width: screen.availableGeometry.width
    height: screen.availableGeometry.height

    property real volume: 1

    function play(uri) {
        dashLoader.item.active = false
        playerLoader.item.playUri(uri)
        playerLoader.focus = true
    }

    Loader {
        id: playerLoader
        source: "player/VideoPlayer.qml"
        anchors.fill: parent
        onLoaded: item.focus = true
    }

    Connections {
        target: playerLoader.item
        onStateChanged: {
            if (playerLoader.item.state == "playing") {
                playerLoader.focus = true
                dashLoader.item.active = false
            } else if (playerLoader.item.state == "stopped") {
                dashLoader.item.active = true
                dashLoader.focus = true
            }
        }
    }

    Image {
        id: wallpaper

        anchors.fill: parent
        source: visible ? "artwork/wallpaper.png" : ""
        visible: unity2dConfiguration.formFactor == "tv"
        fillMode: Image.PreserveAspectCrop
        opacity: playerLoader.item.state == "stopped" ? 1.0 : 0.0
        Behavior on opacity { NumberAnimation { duration: 500; easing.type: Easing.InOutQuad } }
    }

    LauncherLoader {
        id: launcherLoader
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        width: Units.dtPx(launcher2dConfiguration.thickness)
        x: visibilityController.shown ? 0 : -width

        KeyNavigation.right: dashLoader

        Binding {
            target: launcherLoader.item
            property: "showMenus"
            value: !dashLoader.item.active && playerLoader.item.state == "stopped"
        }

        Behavior on x { NumberAnimation { duration: 125 } }

        onLoaded: if (declarativeView.dashActive) {
            launcherLoader.visibilityController.forceVisibleBegin("dash")
        }

        Connections {
            target: declarativeView
            onDashActiveChanged: {
                if (declarativeView.dashActive) launcherLoader.visibilityController.beginForceVisible("dash")
                else launcherLoader.visibilityController.endForceVisible("dash")
            }
        }

        SpreadMonitor {
            id: spread
            enabled: true
            onShownChanged: if (shown) {
                                /* The the spread grabs input and Qt can't properly
                                   detect we've lost input, so explicitly hide the menus */
                                launcherLoader.item.hideMenu()
                                launcherLoader.visibilityController.beginForceVisible("spread")
                            }
                            else launcherLoader.visibilityController.endForceVisible("spread")
        }
    }

    Connections {
        target: launcherLoader.item
        ignoreUnknownSignals: true
        onItemClicked: {
            if (unity2dConfiguration.formFactor == "tv") {
                if (id.match(/\.lens$/)) {
                    dashLoader.item.activateLens(id)
                    dashLoader.focus = true
                } else if (id == "player" && (playerLoader.item.playing || playerLoader.item.paused)) {
                    dashLoader.item.active = false
                    playerLoader.focus = true
                }
            }
        }
    }

    Binding {
        target: launcherLoader.item
        property: "activeComponent"
        value: {
            if ((playerLoader.item.playing || playerLoader.item.paused) && playerLoader.activeFocus) return "player"
            if (dashLoader.item.active) return declarativeView.activeLens
        }
    }

    Loader {
        id: dashLoader
        source: "dash/Dash.qml"
        anchors.top: parent.top
        x: launcherLoader.width
        KeyNavigation.left: launcherLoader
        onLoaded: item.focus = true
        opacity: item.active ? 1.0 : 0.0
        focus: item.active
        Behavior on opacity { NumberAnimation { duration: 500; easing.type: Easing.InOutQuad } }

        Binding {
            target: dashLoader.item
            property: "fullscreenWidth"
            value: screen.availableGeometry.width - launcherLoader.width
        }
    }

    Connections {
        target: declarativeView
        onLauncherFocusRequested: {
            launcherLoader.focus = true
            launcherLoader.item.focusBFB()
        }
        onFocusChanged: {
            /* FIXME: The launcher is forceVisible while it has activeFocus. However even though
               the documentation says that setting focus=false will make an item lose activeFocus
               if it has it, this doesn't happen with FocusScopes (and Launcher is a FocusScope).
               Therefore I'm working around this by giving focus to the shell, which is safe since
               the shell doesn't react to activeFocus at all.
               See: https://bugreports.qt.nokia.com/browse/QTBUG-19688 */
            if (!declarativeView.focus && launcherLoader.activeFocus) shell.focus = true
        }
    }

    Component.onCompleted: declarativeView.show()

    Keys.onPressed: {
        if (event.key == Qt.Key_Escape) {
            switch (unity2dConfiguration.formFactor) {
                case "desktop":
                    declarativeView.forceDeactivateWindow()
                    break
                case "tv":
                    if (!dashLoader.item.active &&
                        (playerLoader.item.playing || playerLoader.item.paused)) {
                        playerLoader.focus = true
                    } else if (playerLoader.item.playing || playerLoader.item.paused) {
                        dashLoader.item.active = false
                        playerLoader.focus = true
                    } else {
                        dashLoader.focus = true
                    }
                    break
            }
        } else if (event.key == Qt.Key_F1 || event.key == Qt.Key_Comma) {
            if (!launcherLoader.activeFocus) {
                launcherLoader.focus = true
            } else {
                if (!dashLoader.item.active &&
                    (playerLoader.item.playing || playerLoader.item.paused)) {
                    playerLoader.focus = true
                } else {
                    dashLoader.focus = true
                }
            }
            event.accepted = true
        /* Forward edge events to currently active component */
        } else if (event.key == Qt.Key_F2 || event.key == Qt.Key_Period || event.key == Qt.Key_F3 || event.key == Qt.Key_MediaPlay) {
            if (!dashLoader.item.active &&
                (playerLoader.item.playing || playerLoader.item.paused)) {
                playerLoader.item.edgeEvent(event)
            } else {
                dashLoader.item.edgeEvent(event)
            }
        }
    }
}
